@@ -497,7 +497,7 @@ extern	char	ccase[];	/*	an array of characters which
 extern	struct	lfile	*filep;	/*	The pointers (lfile *) filep,
 				 *	(lfile *) cfp, and (FILE *) sfp
 				 *	are used in conjunction with
-				 *	the routine getline() to read
+				 *	the routine as_getline() to read
 				 *	asmlnk commands from
 				 *	(1) the standard input or
 				 *	(2) or a command file
@@ -679,7 +679,7 @@ extern	char		get();
 extern	VOID		getfid();
 extern	VOID		getid();
 extern	VOID		getSid();
-extern	int		getline();
+extern	int		as_getline();
 extern	int		getmap();
 extern	char		getnb();
 extern	int		more();
@@ -690,7 +690,7 @@ extern	VOID		chop_crlf();
 /* lkarea.c */
 extern	VOID		lkparea();
 extern	VOID		lnkarea();
-extern	VOID		lnksect();
+extern  Addr_T          lnksect(struct area *tap);
 extern	VOID		newarea();
 
 /* lkhead.c */
