@@ -42,11 +42,6 @@ void Timer(int action, char * message)
 }
 #endif
 
-/* yuck - but including unistd.h causes problems on Cygwin by redefining
- * Addr_T.
- */
-extern int unlink(const char *);
-
 /*)Module   lkmain.c
  *
  *  The module lkmain.c contains the functions which
@@ -92,7 +87,7 @@ void Areas51 (void)
 
     char * rel2[]={
         "XH",
-        "H B areas 0 global symbols",
+        "H C areas 0 global symbols",
         "A _CODE size 0 flags 0",       /*Each .rel has one, so...*/
         "A REG_BANK_0 size 0 flags 4",  /*Register banks are overlayable*/
         "A REG_BANK_1 size 0 flags 4",
@@ -100,6 +95,7 @@ void Areas51 (void)
         "A REG_BANK_3 size 0 flags 4",
         "A BSEG size 0 flags 80",       /*BSEG must be just before BITS*/
         "A BSEG_BYTES size 0 flags 0",  /*Size will be obtained from BSEG in lnkarea()*/
+        "A BIT_BANK size 0 flags 4",    /*Bit register bank is overlayable*/
         "A DSEG size 0 flags 0",
         "A OSEG size 0 flags 4",
         "A ISEG size 0 flags 0",
@@ -154,7 +150,7 @@ void Areas51 (void)
  *
  *  The function main() evaluates the command line arguments to
  *  determine if the linker parameters are to input through 'stdin'
- *  or read from a command file.  The functiond getline() and parse()
+ *  or read from a command file.  The functions as_getline() and parse()
  *  are to input and evaluate the linker parameters.  The linking process
  *  proceeds by making the first pass through each .rel file in the order
  *  presented to the linker.  At the end of the first pass the setbase(),
@@ -208,7 +204,7 @@ void Areas51 (void)
  *      FILE *  afile()     lkmain.c
  *      int     fclose()    c_library
  *      int     fprintf()   c_library
- *      int     getline()   lklex.c
+ *      int     as_getline()   lklex.c
  *      VOID    library()   lklibr.c
  *      VOID    link_main() lkmain.c
  *      VOID    lkexit()    lkmain.c
@@ -289,7 +285,7 @@ main(int argc, char *argv[])
     filep = startp;
     while (1) {
         ip = ib;
-        if (getline() == 0)
+        if (as_getline() == 0)
             break;
         if (pflag && sfp != stdin)
             fprintf(stdout, "%s\n", ip);
@@ -324,7 +320,7 @@ main(int argc, char *argv[])
 
         Areas51(); /*JCF: Create the default 8051 areas in the right order*/
 
-        while (getline()) {
+        while (as_getline()) {
             ip = ib;
 
             /* pass any "magic comments" to NoICE output */
@@ -469,7 +465,7 @@ lkexit(int i)
         copyfile(xfp,dfp);
         fclose(xfp);
         fclose(dfp);
-        unlink("temp.cdb");
+        remove("temp.cdb");
     }*/
     exit(i);
 }
@@ -508,7 +504,7 @@ lkexit(int i)
 VOID
 link_main()
 {
-    register int c;
+    register char c;
 
     if ((c=endline()) == 0) { return; }
     switch (c) {
