@@ -28,7 +28,7 @@
  *		char	endline()
  *		char	get()
  *		VOID	getid(id,c)
- *		int	getline()
+ *		int	as_getline()
  *		int	getmap()
  *		char	getnb()
  *		VOID	getst()
@@ -354,9 +354,9 @@ getmap(d)
 	return (c);
 }
 
-/*)Function	int	getline()
+/*)Function	int	as_getline()
  *
- *	The function getline() reads a line of assembler-source text
+ *	The function as_getline() reads a line of assembler-source text
  *	from an assembly source text file or an include file.
  *	Lines of text are processed from assembler-source files until
  *	all files have been read.  If an include file is opened then
@@ -364,7 +364,7 @@ getmap(d)
  *	include file) until the end of the include file is found.
  *	The input text line is copied into the global string ib[]
  *	and converted to a NULL terminated string.  The function
- *	getline() returns a (1) after succesfully reading a line
+ *	as_getline() returns a (1) after succesfully reading a line
  *	or a (0) if all files have been read.
  *
  *	local variables:
@@ -401,7 +401,7 @@ getmap(d)
  */
 
 int
-getline()
+as_getline()
 {
 register int i;
 
