@@ -524,7 +524,7 @@ extern	VOID		usage();
 extern	char		endline();
 extern	char		get();
 extern	VOID		getid();
-extern	int		getline();
+extern	int		as_getline();
 extern	int		getmap();
 extern	char		getnb();
 extern	VOID		getst();
