@@ -27,7 +27,7 @@ struct	mne	mne[] = {
     { NULL,	"CON",		S_ATYP,		0,	A_CON },
     { NULL,	"OVR",		S_ATYP,		0,	A_OVR },
     { NULL,	"REL",		S_ATYP,		0,	A_REL },
-    { NULL,	"ABS",		S_ATYP,		0,	A_ABS|A_OVR },
+    { NULL,	"ABS",		S_ATYP,		0,	A_ABS },
     { NULL,	"NOPAG",	S_ATYP,		0,	A_NOPAG },
     { NULL,	"PAG",		S_ATYP,		0,	A_PAG },
 
@@ -56,7 +56,7 @@ struct	mne	mne[] = {
     { NULL,	".org",		S_ORG,		0,	0 },
     { NULL,	".module",	S_MODUL,	0,	0 },
     { NULL,	".ascis",	S_ASCIS,	0,	0 },
-    { NULL, ".optsdcc",	S_OPTSDCC,	0,	0},
+    { NULL,	".optsdcc",	S_OPTSDCC,	0,	0 },
 
 	/* z80 / hd64180 */
 
