@@ -22,12 +22,11 @@
    what you give them.   Help stamp out software-hoarding!  
 -------------------------------------------------------------------------*/
 #include "string.h" 
-#define NULL (void *)0
 
 void * memset (
 	void * buf,
 	unsigned char ch ,
-	int count) 
+	size_t count) 
 {
 	register unsigned char * ret = buf;
 
