@@ -48,7 +48,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 COMMAND_DO_WORK_UC(cl_gui_start_cmd)
 {
   class cl_hw *hw;
-  class cl_mem *mem;
+  class cl_memory *mem;
   t_addr start, end;
   class cl_cmd_arg *params[4]= { cmdline->param(0),
 				 cmdline->param(1),
@@ -59,7 +59,7 @@ COMMAND_DO_WORK_UC(cl_gui_start_cmd)
     hw= params[0]->value.hw;
   }
   else if (cmdline->syntax_match(uc, MEMORY ADDRESS ADDRESS)) {
-    mem= params[0]->value.memory;
+    mem= params[0]->value.memory.memory;
     start= params[1]->value.address;
     end= params[2]->value.address;
   }
