@@ -69,11 +69,12 @@ public:
   virtual class cl_uc *get_uc(void) { return(uc); }
 
   // messages from app to handle and broadcast
-  virtual void mem_cell_changed(class cl_mem *mem, t_addr addr);
+  //virtual void mem_cell_changed(class cl_address_space *m, t_addr addr);
   
   virtual int main(void);
   virtual void start(class cl_console *con);
   virtual void stop(int reason);
+  virtual void stop(class cl_ev_brk *brk);
   virtual int step(void);
 };
 
