@@ -1,9 +1,9 @@
 #ifndef ASM_PORT_INCLUDE
 #define ASM_PORT_INCLUDE
 
-void tfprintf (FILE * fp, const char *szFormat,...);
-void tsprintf (char *buffer, const char *szFormat,...);
-void tvsprintf (char *buffer, const char *szFormat, va_list ap);
+void tfprintf (FILE * fp, const char *szFormat, ...);
+void tsprintf (char *buffer, size_t len, const char *szFormat, ...);
+void tvsprintf (char *buffer, size_t len, const char *szFormat, va_list ap);
 
 typedef struct
   {
@@ -26,6 +26,8 @@ struct _ASM_MAPPINGS
  */
 extern const ASM_MAPPINGS asm_asxxxx_mapping;
 extern const ASM_MAPPINGS asm_gas_mapping;
+extern const ASM_MAPPINGS asm_a390_mapping;
+extern const ASM_MAPPINGS asm_xa_asm_mapping;
 
 /** Last entry has szKey = NULL.
  */
@@ -33,4 +35,6 @@ void asm_addTree (const ASM_MAPPINGS * pMappings);
 
 char *FileBaseName (char *fileFullName);
 
+char *printILine (iCode *ic);
+char *printCLine (char *srcFile, int lineno);
 #endif
