@@ -1,10 +1,6 @@
-#define __16F873
-#include "p16f873.h"
-//#include "p16c84.h"
+#include "gpsim_assert.h"
 
-unsigned char success=0;
 unsigned char failures=0;
-unsigned char dummy=0;
 
 bit bit0 = 0;
 bit bit1 = 0;
@@ -13,15 +9,13 @@ unsigned int aint1 = 0;
 unsigned char achar0 = 0;
 unsigned char achar1 = 0;
 
-void done()
+void
+done()
 {
-
-  dummy++;
-
+  ASSERT(MANGLE(failures) == 0);
+  PASSED();
 }
 
-
-
 void bit_invert(void)
 {
 
@@ -46,6 +40,5 @@ void main(void)
   bit_invert();
   bit_copy();
 
-  success = failures;
   done();
 }
